#include <string.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>

#include <gtk/gtk.h>

#include <libintl.h>
#define _(String) gettext(String)

#include <libxml/parser.h>

#include "window.h"
#include "lyrc.h"
#include "lyrc_site.h"
#include "globals.h"
#include "filetype.h"
#include "graphics.h"

// void update_zoom( int how );
gint max( gint a, gint b );

gchar *multi_fname = NULL;
gboolean search_on = FALSE;

void lyric_window( gchar * filename )
{
	gint counter = 0;
	gchar *source = NULL;
	gchar *song = NULL;
	gchar *message = NULL;
	gchar *artist = NULL;
	gchar *temp;
	gchar *line;
	gint i = 0;
	gint oldlen;
	
	temp = lyrics;
	//lyrics = strdup( "" );
	lyrics = (char*) calloc( 1, sizeof(char) );
	g_free( temp );
	temp = lyrics_nobr;
	//lyrics_nobr = strdup( "" );
	lyrics_nobr = (char*) calloc( 1, sizeof(char) );
	g_free( temp );
	
	if( multi_fname ) {
		free( multi_fname );
		multi_fname = NULL;
	}
	if( filename && strstr( filename, ".xml" ) ) {	/* xml file */
		xmlDocPtr doc;
		xmlNodePtr node, node2;
		while( ( doc = xmlParseFile( filename ) ) == NULL ) {
			// retry 2 times in case it's looooong and just being written
			if( i++ > 2 ) {
				// file can't be parsed: avoid crash
				lyrics = g_strdup_printf( "\t\n%s\n%s\n%s\n%s\n",
										  _( "This lyrics file is invalid:" ),
										  filename,
										  "---------------------------------------------------------------------------------------------------------------------------------",
										  "[MANAGE]" );
				lyrics_offset = 0;
				lyrics_offset_h = 0;
				coview_draw_lyrics(  );
				return;
			} else {
				sleep( 1 );
			}
		}
		node = xmlDocGetRootElement( doc );
		if( !xmlStrcmp( node->name, ( const xmlChar * ) "lyrc" ) ) {
			node = node->xmlChildrenNode;
			if( !xmlStrcmp( node->name, ( const xmlChar * ) "result" ) ) {
				/* Result lyrics */
				node = node->xmlChildrenNode;
				while( node != NULL ) {
					if( !xmlStrcmp( node->name, ( const xmlChar * ) "lyric" ) ) {
						source = xmlNodeListGetString( doc, node->xmlChildrenNode, 1 );
						temp = utf8_decode( source );
						source = g_strdup_printf( "%s\n", temp );
						lyrics_nobr = g_strdup( temp );
						free( temp );
					}
					node = node->next;
				}

				// INSERT LINE BREAKS -------------------------------------- >>>
				temp = source;

				do {
					line = nextline( &temp );
					i = lyrics_maxlinelength - 1;

					while( strlen( line ) > lyrics_maxlinelength ) {
						while( line[i] != '.' &&
							   line[i] != '?' &&
							   line[i] != '!' &&
							   line[i] != '\"' &&
							   line[i] != ' ' &&
							   i > 0 ) {
							i--;
						}
						oldlen = strlen( lyrics );
						lyrics = ( char * ) realloc( lyrics, oldlen + i + 3 );
						lyrics = strncat( lyrics, line, i+1 );
						lyrics[oldlen + i + 1] = '\n';
						lyrics[oldlen + i + 2] = '\0';

						line += i + 1;
						i = lyrics_maxlinelength - 1;
					}
					lyrics = ( char * ) realloc( lyrics, strlen( line ) + strlen( lyrics ) + 2 );
					lyrics = strcat( lyrics, line );
					lyrics = strcat( lyrics, "\n" );
				}
				while( line != temp );

				if( lyrics[0] != '\n' ) {
					temp = lyrics;
					lyrics = g_strdup_printf( "\n%s", lyrics );
					free( temp );
				}
				temp = lyrics;
				lyrics = strip_end_spaces( lyrics );
				free( temp );
				temp = lyrics;
				lyrics = g_strdup_printf( "%s\n", lyrics );
				free( temp );
				// <<< --------------------------------------------------------

				g_free( source );

			} else if( !xmlStrcmp( node->name, ( const xmlChar * ) "suggest" ) ) {
				/* multiple choice */
				multi_fname = strdup( filename );
				lyrics = g_strdup_printf( "\t\n%s %d/%d.\n%s\n%s",
										  _( "Lyrics could not be retrieved from source" ), script_nr+1, NUM_SCRIPTS,
										  _( "Please check the following list:" ),
										  "---------------------------------------------------------------------------------------------------------------------------------" );
				node = node->xmlChildrenNode;

				while( node != NULL ) {
					if( !xmlStrcmp( node->name, ( const xmlChar * ) "message" ) ) {
						message = xmlNodeListGetString( doc, node->xmlChildrenNode, 1 );
						message = utf8_decode( message );
						g_free( lyrics );
						lyrics = g_strdup_printf( "\t\n%s %d/%d.\n%s", _( "Lyrics could not be retrieved from source" ), script_nr+1, NUM_SCRIPTS, message );
					        free( message );
					}
					if( !xmlStrcmp( node->name, ( const xmlChar * ) "searchOn" ) ) {
					        search_on = TRUE;
					}
					if( !xmlStrcmp( node->name, ( const xmlChar * ) "song" ) ) {
						/* here is another song */
						counter++;
						node2 = node->xmlChildrenNode;
						while( node2 != NULL ) {
							if( !xmlStrcmp( node2->name, ( const xmlChar * ) "name" ) ) {
								song = xmlNodeListGetString( doc, node2->xmlChildrenNode, 1 );
								song = utf8_decode( song );
							}
							if( !xmlStrcmp( node2->name, ( const xmlChar * ) "group" ) ) {
								artist = xmlNodeListGetString( doc, node2->xmlChildrenNode, 1 );
								artist = utf8_decode( artist );
							}
							node2 = node2->next;
						}
						lyrics = g_strdup_printf( "%s\n%s", lyrics, g_strdup_printf( "%s - %s", artist, song ) );
						free( artist );
						free( song );
					}
					node = node->next;
				}

				lyrics = g_strdup_printf( "%s\n%s\n%s%s\n%s\n",
							  lyrics,
							  "---------------------------------------------------------------------------------------------------------------------------------",
							  ( search_on ? "[search goes on... please wait]\n" :
							    ( script_nr < NUM_SCRIPTS - 1 ? "[SEARCH NEXT SOURCE!]\n" : "" )
							    ),
							  "[SEARCH THE WEB]", "[MANAGE]" );
			}
		}
		xmlFreeDoc( doc );
		lyrics_offset = 0;
		lyrics_offset_h = 0;
		coview_load_pic_index(  );
		coview_draw_lyrics(  );
	        if( search_on ) {
		   search_on = FALSE;
		   get_lyrics( play_artist, play_song, NULL, NULL, 0, ++script_nr );
		}
		return;
	}
	if( filename ) {
		if( strcmp( filename, "LOOK" ) == 0 ) {	/* Looking page */

			lyrics = g_strdup_printf( "\t\n%s\n%s\n",
									  _( "Looking for the lyrics." ),
									  _( "Please wait..." ) );

		} else {
			if( fsize( filename ) == 0 ) {
				/* No result */
				lyrics = g_strdup_printf( "\t\n%s\n%s\n%s\n%s\n%s\n",
										  _( "Sorry, the lyrics weren't found." ),
										  _( "If you find them manually, please upload." ),
										  "---------------------------------------------------------------------------------------------------------------------------------",
										  "[SEARCH THE WEB]", "[MANAGE]" );

			} else {
				/* advance html page */
				lyrics = g_strdup_printf( "\t\n%s\n%s\n%s\n",
										  _( "Sorry, the lyrics weren't found" ),
										  _( "maybe you'll find them with this..." ), filename );
			}
		}
	} else {
		/* Not enough information */
		lyrics = g_strdup_printf( "\t\n%s\n%s\n",
								  _( "Sorry, the lyrics can't be retrieved." ),
								  _( "There's not enough information about this file." ) );
	}

	lyrics_offset = 0;
	lyrics_offset_h = 0;
	coview_load_pic_index(  );
	coview_draw_lyrics(  );
}

void multi_choice( gint nb )
{
	gint counter = 0;
	xmlDocPtr doc;
	xmlNodePtr node;
	gchar *artist, *song, *rartist = NULL, *rsong = NULL;
	if( !multi_fname )
		return;
	doc = xmlParseFile( multi_fname );
	node = xmlDocGetRootElement( doc );	/* point to lyrc */
	node = node->xmlChildrenNode;
	while( xmlStrcmp( node->name, ( const xmlChar * ) "suggest" ) )
		node = node->next;		/* point to suggest */
	node = node->xmlChildrenNode;
	while( node != NULL ) {
		if( !xmlStrcmp( node->name, ( const xmlChar * ) "song" ) )
			counter++;
		if( counter < nb )
			node = node->next;
		else
			break;
	}							/* pointing to the good song */

	if( node == NULL )
		return;
	node = node->xmlChildrenNode;
	while( node != NULL ) {
		if( !xmlStrcmp( node->name, ( const xmlChar * ) "name" ) ) {
			rsong = xmlNodeListGetString( doc, node->xmlChildrenNode, 1 );
			rsong = utf8_decode( rsong );
		}
		if( !xmlStrcmp( node->name, ( const xmlChar * ) "group" ) ) {
			rartist = xmlNodeListGetString( doc, node->xmlChildrenNode, 1 );
			rartist = utf8_decode( rartist );
		}
		node = node->next;
	}

	artist = ( gchar * ) strdup( cur_file_info->artist );
	song = ( gchar * ) strdup( cur_file_info->title );
	artist = ( gchar * ) net_purge( artist );
	artist = ( gchar* ) strip_end_spaces( artist );
	song = ( gchar * ) net_purge( song );
	song = ( gchar* ) strip_end_spaces( song );
	g_strdown( artist );
	g_strdown( song );
/*	rartist=(gchar *) net_purge(rartist);
	rsong=(gchar *) net_purge(rsong);*/

	delete_lyrics(  );
	get_lyrics( play_artist, play_song, rartist, rsong, 1, script_nr );

	flag = FALSE;
	free( artist );
	free( song );
	free( rartist );
	free( rsong );
}

long fsize( gchar * filename )
{
	FILE *f;
	long len;
	f = fopen( filename, "r" );
	fseek( f, 0, SEEK_END );
	len = ftell( f );
	fclose( f );
	return len;
}

int file_age( gchar * filename )
{
	struct stat buf;
	struct timeval time_now;
	gettimeofday( &time_now, NULL );
	if( stat( filename, &buf ) == -1 )
		return INT_MAX;
	else
		return time_now.tv_sec - buf.st_ctime;

}

// void update_zoom( int how )
// {
//  gchar *fontname;
//  gchar *font_format = "-b&h-lucida-medium-r-normal-*-%u-*-*-*-*-*-*-*";
//  gchar *font_title_format = "-b&h-lucida-bold-r-normal-*-%u-*-*-*-*-*-*-*";
// 
//  font = NULL;
//  switch ( how ) {
//  case 0:
//      zoom_level++;
//      while( !font ) {
//          fontsize += 1;
//          fontname = g_strdup_printf( font_format, fontsize );
//          font = gdk_fontset_load( fontname );
//          g_free( fontname );
//          fontname = g_strdup_printf( font_title_format, fontsize );
//          font_title = gdk_fontset_load( fontname );
//          g_free( fontname );
//      }
//      flag = FALSE;
//      return;
//  case 1:
//      zoom_level--;
//      while( !font ) {
//          fontsize -= 1;
//          if( fontsize < 2 ) {
//              fontsize = 2;
//              zoom_level++;
//          }
//          fontname = g_strdup_printf( font_format, fontsize );
//          font = gdk_font_load( fontname );
//          g_free( fontname );
//          fontname = g_strdup_printf( font_title_format, fontsize );
//          font_title = gdk_fontset_load( fontname );
//          g_free( fontname );
//      }
//      flag = FALSE;
//      return;
//  case 2:
//      zoom_level = 0;
//      fontsize = 11;
//      while( !font ) {
//          fontsize++;
//          fontname = g_strdup_printf( font_format, fontsize );
//          font = gdk_font_load( fontname );
//          g_free( fontname );
//          fontname = g_strdup_printf( font_title_format, fontsize );
//          font_title = gdk_fontset_load( fontname );
//          g_free( fontname );
//      }
//      flag = FALSE;
//      return;
//  }
// }

gboolean about_on = FALSE;

void about_exit( GtkWidget * win )
{
	about_on = FALSE;
	gtk_widget_destroy( win );
}

gint get_width( gchar * str, GdkFont * font )
{
	gchar *str_temp, *ptr, *pos;
	gint temp_len, len = 0;

	if( !str )
		return 0;
	str_temp = strdup( str );
	ptr = str_temp;

	while( ( pos = strstr( ptr, "\n" ) ) ) {
		pos[0] = 0;
		temp_len = gdk_text_width( font, ptr, strlen( ptr ) ) + 6;
		if( temp_len > len )
			len = temp_len;
		ptr = pos + 1;
	}
	temp_len = gdk_text_width( font, ptr, strlen( ptr ) ) + 6;
	if( temp_len > len )
		len = temp_len;
	free( str_temp );
	return len;
}

gint get_height( gchar * str, GdkFont * font )
{
	gchar *str_temp, *ptr, *pos;
	gint len = 0;

	if( !str )
		return 0;
	str_temp = strdup( str );
	ptr = str_temp;

	while( ( pos = strstr( ptr, "\n" ) ) ) {
		len += font->ascent + font->descent;
		ptr = pos + 1;
	}
	len += font->ascent + font->descent;
	free( str_temp );
	return len;
}

gint max( gint a, gint b )
{
	if( a > b )
		return a;
	return b;
}
